﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace IRDemo
{
	public partial class MainForm : Form
	{
		const int FLAG_CONTROL = 0x4354524c;

		const int CMD_ONOFF = 0;
		const int CMD_WORKMODE = 1;

		const int VALUE_ON = 1;
		const int VALUE_OFF = 0;

		const int WORKMODE_38K = 0;				//38K远距离低精度
		const int WORKMODE_WIDE = 1;			//宽频近距离高精度
		const int WORKMODE_NOCARRIER = 2;	//无载波近距离高精度
		const int WORKMODE_CARRIER = 3;		//载波测试
		const int WORKMODE_56K = 4;				//56K远距离低精度(YG-920 V10.0 和YG-2210B支持)

		[StructLayout(LayoutKind.Sequential)]
		public struct IR_DATA
		{
			public int type;							//0-->普通遥控,1--->空调遥控器,2--->自定义
			public ulong sys_code;				//客户码(空调遥控器和自定义遥控器无客户码)
			public ulong data_code;			//键码(空调遥控器和自定义遥控器无键码)
			public int sys_len;						//客户码位数(空调遥控器和自定义遥控器固定为0)
			public int data_len;					//键码位数(空调遥控器和自定义遥控器固定为0)
			public int code_time;				//码长
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 50)]
			public string format;				//格式名称
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 255)]
			public string full_code;				//全码
			public int wave_len;					//波形长度
			[MarshalAs(UnmanagedType.ByValArray, SizeConst = 2048)]
			public uint[] wave_buf;				//波形数据
		}

		string[] remote_type = { "普通遥控器", "空调遥控器", "自定义遥控器" };

		public MainForm()
		{
			InitializeComponent();
		}

		protected override void DefWndProc(ref Message m)
		{
			switch ( m.Msg )
			{
				case Win32Func.WM_COPYDATA:
				{
					var ds = (COPYDATASTRUCT)m.GetLParam(typeof(COPYDATASTRUCT));
					var ir = (IR_DATA)Marshal.PtrToStructure((IntPtr)ds.lpData, typeof(IR_DATA));

					this.txbType.Text = remote_type[ir.type];
					this.txbFormat.Text = ir.format;
					this.txbClient.Text = (ir.sys_len == 0) ? string.Empty : ir.sys_code.ToString("X");
					this.txbCommand.Text = (ir.data_len == 0) ? string.Empty : ir.data_code.ToString("X");
					this.txbCode.Text = ir.full_code;
					this.txbCodeTime.Text = ir.code_time.ToString();

					var lst = new List<string>();
					for ( int i = 0 ; i < ir.wave_len ; ++i ) lst.Add(ir.wave_buf[i].ToString());
					this.txbWave.Text = string.Join(Environment.NewLine, lst.ToArray());
					break;
				}
				default:
				{
					base.DefWndProc(ref m);
					break;
				}
			}
		}

		private IntPtr MyFindWindow()
		{
			IntPtr hPrevWnd, hData;
			Win32Func.CreateMutex(IntPtr.Zero, true, "IROffice");
			if ( Marshal.GetLastWin32Error() == Win32Func.ERROR_ALREADY_EXISTS )
			{
				// ShowMessage ("本仪器数据采集程序已运行！") ;
				hPrevWnd = Win32Func.GetDesktopWindow();
				hPrevWnd = Win32Func.GetWindow(hPrevWnd, GetWindow_Cmd.GW_CHILD);
				while ( Win32Func.IsWindow(hPrevWnd) )
				{
					// 判断此窗口属性标志是否与设置的主窗口或登录窗口标志相符.
					hData = Win32Func.GetProp(hPrevWnd, "IROffice");
					if ( hData == (IntPtr)1 ) return hPrevWnd;  // 找到<红外遥控管理系统>
					hPrevWnd = Win32Func.GetWindow(hPrevWnd, GetWindow_Cmd.GW_HWNDNEXT);
				}
			}
			Win32Func.CreateMutex(IntPtr.Zero, true, "IRReader");
			if ( Marshal.GetLastWin32Error() == Win32Func.ERROR_ALREADY_EXISTS )
			{
				// ShowMessage ("本仪器数据采集程序已运行！") ;
				hPrevWnd = Win32Func.GetDesktopWindow();
				hPrevWnd = Win32Func.GetWindow(hPrevWnd, GetWindow_Cmd.GW_CHILD);
				while ( Win32Func.IsWindow(hPrevWnd) )
				{
					// 判断此窗口属性标志是否与设置的主窗口或登录窗口标志相符.
					hData = Win32Func.GetProp(hPrevWnd, "IRReader");
					if ( hData == (IntPtr)1 ) return hPrevWnd;  // 找到<红外遥控编码分析仪>
					hPrevWnd = Win32Func.GetWindow(hPrevWnd, GetWindow_Cmd.GW_HWNDNEXT);
				}
			}
			return IntPtr.Zero;
		}

		private void btnOpen_Click(object sender, EventArgs e)
		{
			COPYDATASTRUCT ds;
			var buf = new int[2];
			IntPtr destWin = this.MyFindWindow();
			if ( destWin == IntPtr.Zero )
			{
				MessageBox.Show(this, "红外遥控编码分析仪没有运行.");
				return;
			}
			buf[0] = CMD_ONOFF;
			buf[1] = VALUE_ON;

			GCHandle handle = GCHandle.Alloc(buf, GCHandleType.Pinned);
			ds.dwData = (IntPtr)FLAG_CONTROL;
			ds.cbData = sizeof(int) * buf.Length;
			ds.lpData = handle.AddrOfPinnedObject();

			var msgResult = Win32Func.SendMessage(destWin, Win32Func.WM_COPYDATA, this.Handle, ref ds);
			this.toolStripStatusLabel1.Text = msgResult != 0 ? "打开成功:)" : "打开失败:(";
		}

		private void btnClose_Click(object sender, EventArgs e)
		{
			COPYDATASTRUCT ds;
			var buf = new int[2];
			IntPtr destWin = this.MyFindWindow();
			if ( destWin == IntPtr.Zero )
			{
				MessageBox.Show(this, "红外遥控编码分析仪没有运行.");
				return;
			}
			buf[0] = CMD_ONOFF;
			buf[1] = VALUE_OFF;

			GCHandle handle = GCHandle.Alloc(buf, GCHandleType.Pinned);
			ds.dwData = (IntPtr)FLAG_CONTROL;
			ds.cbData = sizeof(int) * buf.Length;
			ds.lpData = handle.AddrOfPinnedObject();

			var msgResult = Win32Func.SendMessage(destWin, Win32Func.WM_COPYDATA, this.Handle, ref ds);
			this.toolStripStatusLabel1.Text = msgResult != 0 ? "关闭成功:)" : "关闭失败:(";
		}

		private void rbtnWorkMode_CheckedChanged(object sender, EventArgs e)
		{
			COPYDATASTRUCT ds;
			var buf = new int[2];
			IntPtr destWin = this.MyFindWindow();
			if ( destWin == IntPtr.Zero )
			{
				MessageBox.Show(this, "红外遥控编码分析仪没有运行.");
				return;
			}
			buf[0] = CMD_WORKMODE;
			buf[1] = (sender as RadioButton).TabIndex;

			GCHandle handle = GCHandle.Alloc(buf, GCHandleType.Pinned);
			ds.dwData = (IntPtr)FLAG_CONTROL;
			ds.cbData = sizeof(int) * buf.Length;
			ds.lpData = handle.AddrOfPinnedObject();

			var msgResult = Win32Func.SendMessage(destWin, Win32Func.WM_COPYDATA, this.Handle, ref ds);
			this.toolStripStatusLabel1.Text = msgResult != 0 ? "设置采样器工作模式成功:)" : "设置采样器工作模式失败:(";
		}
	}
}
